@@ -17,12 +17,12 @@ the AUTHORS file
 
 =cut
 
+use 5.010;
 use Data::Dumper;
 use File::Find;
 use Getopt::Long;
 use strict;
-
-sub findauthors;
+use Pod::Simple::SimpleTree;
 
 #
 # command line options
@@ -84,6 +84,7 @@ my @dirs = qw( ../Bio/ ../scripts . );
 # Read the AUTHORS file
 #
 
+my $parser = Pod::Simple::SimpleTree->new();
 
 open my $F, '<', $authorsfile or die "Could not read file '$authorsfile': $!\n";
 
@@ -124,33 +125,40 @@ print Dumper \%NEWAUTHORS;
 # this is where the action is
 #
 sub findauthors {
-    my ($filename) = @_;
+    my ($filename) = $_;
     return unless ($filename =~ /\.PLS$/ or $filename =~ /\.p[ml]$/);
     return unless -e $filename;
-
-    print "$filename\n" if $verbose;
-    #local $/=undef;
-    open my $F, '<', $filename or die "Could not read file '$filename': $!\n";
-    while (my $line = <$F>) {
-        #print $line;
-        last if $line =~ /=head1 +AUTHOR/;
-    }
-    my $authorblock;
-    while (my $line = <$F>) {
-        last if $line =~ /=head/ and $line !~ /CONTRIBUTORS/;
-        $authorblock .= $line;
-    }
-    close $F;
-    return unless $authorblock;
-
-    while ( $authorblock =~ /([\.\w_-]+@[\.a-z_-]+)/g) {
-        #my $email = $1;
-        #$email =~ //
-        next if $AUTHORS{$1};
-        #print "$filename\t$1\n";
-
-        push @{$NEWAUTHORS{$1}}, $filename;
+    
+    my $root = $parser->parse_file($filename)->root();
+    #say Dumper $tree->root();
+    
+    #print "$filename\n" if $verbose;
+    ##local $/=undef;
+    #open my $F, '<', $filename or die "Could not read file '$filename': $!\n";
+    for my $chunk (@$root) {
+        next unless ref $chunk eq 'ARRAY';
+        if ($chunk->[0] eq 'head1') {
+            print Dumper $chunk
+        }
+    #    #print $line;
+    #    last if $line =~ /=head1 +AUTHOR/;
     }
+    #my $authorblock;
+    #while (my $line = <$F>) {
+    #    last if $line =~ /=head/ and $line !~ /CONTRIBUTORS/;
+    #    $authorblock .= $line;
+    #}
+    #close $F;
+    #return unless $authorblock;
+    #
+    #while ( $authorblock =~ /([\.\w_-]+@[\.a-z_-]+)/g) {
+    #    #my $email = $1;
+    #    #$email =~ //
+    #    next if $AUTHORS{$1};
+    #    #print "$filename\t$1\n";
+    #
+    #    push @{$NEWAUTHORS{$1}}, $filename;
+    #}
 }
 
 
